#include <avr/interrupt.h>
#include <avr/io.h>//it includes <avr/sfr_defs.h>, <avr/portpins.h>, <avr/common.h>, <avr/version.h>
#include <stdint.h>

#include "pwm_tmr2.h"
#include "tmr0.h"
#include "utils.h"

// parameters of the filer to oscillate at Fa=600Hz
#define  A1_t -503
#define  A2_t  256
#define  b0_t  19

// amplitude of the oscillation
#define  A    100
// DC component to add before sending to PWM 
#define  DC  A+1
 
/********************************************
  Digital oscillator (with 8-bit variables)
---------------------------------------------

This code computes samples of a sinusoidal 
signal at a sampling rate Fs using a second order
filter.
These samples update the PWM output and
generate a sinusoidal signal Fa.

-pwm_tmr2 module
With the output of the filter
it updates the PWM output at terminal ~3 or ~11.

-tmr0 module
An interrupt is activated each 50us 
(ocr0a=99, tmr0_pre=8)

-debug output
The digital output PD4 is set to '1' 
at the beginning of the ISR and to '0' at the end.

November 15, 2019
*********************************************/
  void setup(){
  // tmr0
  setup_tmr0(99,8);//(ocr0a, tmr0_pre)
  //tmr0_pre 1,default=8,64,256,1024
  //TMR0=prescaler*(ocr0a+1)*T_clk

  // pwm
  setup_pwm_tmr2(11);//(pwm_out) 3,default=11
  set_prescaler_tmr2(1);
  
  // debug output
  sbi(DDRD,DDD4);//DDRD |=(1<<DDD4);//pin 4 Arduino as an output. It shows sampling period (period) and ISR execution time (pulse wide)
  sbi(DDRB,DDB5);//pin 13 (LED) Arduino as an output. It shows if one or more times the conditin yn>A or yn<-A has been fulfilled
  
  sei();
}

int main(void){
  setup();
  while(1){}
}

ISR(TIMER0_COMPA_vect){
  static int8_t yn8_1=b0_t,yn8_2=0;
  sbi(PORTD,PD4);
  
  // filter computation
  int8_t yn8=(-A1_t*yn8_1-A2_t*yn8_2)>>8;
  //yn8=(-A1_t*yn8_1)>>8;yn8=yn8-yn8_2;//only if A2_t=256 (r=1), then CPU time is 3.2us
  
  // this IF is necessary to avoid instability
  if (yn8>A){
    yn8=A;
    sbi(PORTB,PB5);
  }
  else if (yn8<-A) {
    yn8=-A;
    sbi(PORTB,PB5);    
  }
  
  // saving the memory
  yn8_2=yn8_1;
  yn8_1=yn8;
  
  // PWM actualization
  set_pwm_tmr2(yn8+DC,11);  

  cbi(PORTD,PD4);
}
